import 'package:flutter/material.dart';
import 'package:dialer/features/contacts/contact_page.dart';
import 'package:dialer/features/favorites/favorites_page.dart';
import 'package:dialer/features/history/history_page.dart';
import 'package:dialer/features/composition/composition.dart';
import 'package:flutter_contacts/flutter_contacts.dart';
import 'package:dialer/features/settings/settings.dart';
import '../../services/contact_service.dart';

const bool _privacyMode = bool.fromEnvironment('privacy-mode', defaultValue: false);

String _maskName(String name) {
  if (!_privacyMode) return name;
  final parts = name.split(' ');
  return parts.map((part) {
    if (part.length < 2) return part;
    return '${part[0]}${'*' * (part.length - 1)}';
  }).join(' ');
}

class _MyHomePageState extends State<MyHomePage>
    with SingleTickerProviderStateMixin {
  late TabController _tabController;
  List<Contact> _allContacts = [];
  List<Contact> _contactSuggestions = [];
  final ContactService _contactService = ContactService();


  @override
  void initState() {
    super.initState();
    // Set the TabController length to 3
    _tabController = TabController(length: 3, vsync: this, initialIndex: 1);
    _tabController.addListener(_handleTabIndex);
    _fetchContacts();
  }

  void _fetchContacts() async {
    _allContacts = await _contactService.fetchContacts();
    setState(() {});
  }

  void _onSearchChanged(String query) {
    print("Search query: $query");

    setState(() {
      if (query.isEmpty) {
        _contactSuggestions = List.from(_allContacts);
      } else {
        _contactSuggestions = _allContacts.where((contact) {
          return contact.displayName
              .toLowerCase()
              .contains(query.toLowerCase());
        }).toList();
      }
    });
  }

  @override
  void dispose() {
    _tabController.removeListener(_handleTabIndex);
    _tabController.dispose();
    super.dispose();
  }

  void _handleTabIndex() {
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.black,
      body: Column(
        children: [
          // Persistent Search Bar
          Padding(
            padding: const EdgeInsets.only(
              top: 24.0,
              bottom: 10.0,
              left: 16.0,
              right: 16.0,
            ),
            child: Row(
              children: [
                Expanded(
                  child: Container(
                    decoration: BoxDecoration(
                      color: const Color.fromARGB(255, 30, 30, 30),
                      borderRadius: BorderRadius.circular(12.0),
                      border: Border(
                        top: BorderSide(color: Colors.grey.shade800, width: 1),
                        left: BorderSide(color: Colors.grey.shade800, width: 1),
                        right: BorderSide(color: Colors.grey.shade800, width: 1),
                        bottom:
                        BorderSide(color: Colors.grey.shade800, width: 2),
                      ),
                    ),
                    child: SearchAnchor(
                      builder:
                          (BuildContext context, SearchController controller) {
                        return SearchBar(
                          controller: controller,
                          padding:
                          MaterialStateProperty.all<EdgeInsetsGeometry>(
                            const EdgeInsets.only(
                              top: 6.0,
                              bottom: 6.0,
                              left: 16.0,
                              right: 16.0,
                            ),
                          ),
                          onTap: () {
                            controller.openView();
                            _onSearchChanged('');
                          },
                          backgroundColor: MaterialStateProperty.all(
                              const Color.fromARGB(255, 30, 30, 30)),
                          hintText: 'Search contacts',
                          hintStyle: MaterialStateProperty.all(
                            const TextStyle(color: Colors.grey, fontSize: 16.0),
                          ),
                          leading: const Icon(
                            Icons.search,
                            color: Colors.grey,
                            size: 24.0,
                          ),
                          shape:
                          MaterialStateProperty.all<RoundedRectangleBorder>(
                            RoundedRectangleBorder(
                              borderRadius: BorderRadius.circular(12.0),
                            ),
                          ),
                        );
                      },
                      viewOnChanged: (query) {
                        _onSearchChanged(query);
                      },
                      suggestionsBuilder:
                          (BuildContext context, SearchController controller) {
                        return _contactSuggestions.map((contact) {
                          return _buildSuggestionTile(contact, controller);
                        }).toList();
                      },
                    ),
                  ),
                ),
                // 3-dot menu
                PopupMenuButton<String>(
                  icon: const Icon(Icons.more_vert, color: Colors.white),
                  itemBuilder: (BuildContext context) => [
                    const PopupMenuItem<String>(
                      value: 'settings',
                      child: Text('Settings'),
                    ),
                  ],
                  onSelected: (String value) {
                    if (value == 'settings') {
                      Navigator.push(
                        context,
                        MaterialPageRoute(
                            builder: (context) => const SettingsPage()),
                      );
                    }
                  },
                ),
              ],
            ),
          ),
          // Main content with TabBarView
          Expanded(
            child: Stack(
              children: [
                TabBarView(
                  controller: _tabController,
                  children: const [
                    FavoritesPage(),
                    HistoryPage(),
                    ContactPage(),
                  ],
                ),
                Positioned(
                  right: 20,
                  bottom: 20,
                  child: FloatingActionButton(
                    onPressed: () {
                      Navigator.push(
                        context,
                        MaterialPageRoute(
                          builder: (context) => const CompositionPage(),
                        ),
                      );
                    },
                    backgroundColor: Colors.blue,
                    shape: RoundedRectangleBorder(
                      borderRadius: BorderRadius.circular(45),
                    ),
                    child: const Icon(Icons.dialpad, color: Colors.white),
                  ),
                ),
              ],
            ),
          ),
        ],
      ),
      bottomNavigationBar: Container(
        color: Colors.black,
        child: TabBar(
          controller: _tabController,
          tabs: [
            Tab(
                icon: Icon(_tabController.index == 0
                    ? Icons.star
                    : Icons.star_border)),
            Tab(
                icon: Icon(_tabController.index == 1
                    ? Icons.access_time_filled
                    : Icons.access_time_outlined)),
            Tab(
                icon: Icon(_tabController.index == 2
                    ? Icons.contacts
                    : Icons.contacts_outlined)),
          ],
          labelColor: Colors.white,
          unselectedLabelColor: const Color.fromARGB(255, 158, 158, 158),
          indicatorSize: TabBarIndicatorSize.label,
          indicatorColor: Colors.white,
        ),
      ),
    );
  }

  Widget _buildSuggestionTile(Contact contact, SearchController controller) {
    final maskedName = _maskName(contact.displayName);
    return ListTile(
      key: ValueKey(contact.id),
      title: Text(maskedName, style: const TextStyle(color: Colors.white)),
      onTap: () {
        controller.closeView(maskedName);
      },
    );
  }
}

class MyHomePage extends StatefulWidget {
  const MyHomePage({super.key});

  @override
  _MyHomePageState createState() => _MyHomePageState();
}
